/**************************************************************/
/*  IR PTT Controler                                          */
/*                                                            */
/*  Base sketch by DFROBOT Sample Code                        */
/*  http://www.dfrobot.com/wiki/index.php/IR_Kit(SKU:DFR0107) */
/*                                                            */
/*  Author:    Hiroshi Shirahara(7J3AOZ)                      */
/*  Date:      2014/07/01                                     */
/*                                                            */
/*  This code is in the public domain.                        */
/*                                                            */
/**************************************************************/

#include <Metro.h>                        // 「Metro」ライブラリのヘッダーファイル組み込み

#define PTT_ON 1                          // PTTオンを示す定数
#define PTT_OFF 0                         // PTTオフを示す定数

#define IR_BIT_LENGTH 32                  // 赤外線リモコンの送出バイト数
#define BIT_1 1500                        // 1を示す赤外線パルス長（マイクロ秒）
#define BIT_0 450                         // 0を示す赤外線パルス長（マイクロ秒）
#define BIT_START 4000                    // スタートビットを示す赤外線パルス長（マイクロ秒）

#define VERIFY_CODE 0x3F00                // カスタムコード
#define KEY0_CODE  0xF30C                 // リモコンのキー0
 
#define SW_PIN 2                          // スイッチが接続されている入出力ピン番号
#define IR_PIN 7                          // 赤外線センサーが接続されている入出力ピン番号

#define LED_PIN 11                        // LEDが接続されている入出力ピン番号
#define BUZZ_PIN 12                       // ブザーが接続されている入出力ピン番号
#define PTT_PIN 13                        // PTT制御用フォトカプラが接続されている入出力ピン番号

#define ALERM_TIME 300                    // PTTの警告時間（秒）
#define FORCE_OFF_TIME 600                // PTTの強制オフ時間（秒）

int ptt_timer;                            // PTTオンタイマー
volatile int state;                       // 動作ステータス

Metro timer1 = Metro(1000);               //Metroのインスタンス生成

// ------ 初期化

void setup() {
  
  ptt_timer = 0;
  state = PTT_OFF;

  pinMode(SW_PIN, INPUT_PULLUP);                   // スイッチが接続されているピンを入力（内部プルアップ）に設定
  attachInterrupt(0,button_in,FALLING);            // スイッチが接続されているピンによる割り込みを設定

  pinMode(IR_PIN, INPUT);                          // 赤外線センサが接続されているピンを入力に設定

  pinMode(LED_PIN, OUTPUT);                        // LEDが接続されているピンを出力に設定
  pinMode(BUZZ_PIN, OUTPUT);                       // ブザーが接続されているピンを出力に設定
  pinMode(PTT_PIN, OUTPUT) ;                       // フォトカプラが接続されているピンを出力に設定 

  ptt_control();                                   // PTT制御（PTTとLEDをオフ）

  tone(BUZZ_PIN,400,100);                          // 起動を示すためにブザーを3回鳴らす
  delay(200);
  tone(BUZZ_PIN,400,100);
  delay(200);
  tone(BUZZ_PIN,400,100);

}

// ------ メインループ

void loop() {

  int ir_code;                                     // 受信した赤外線コード

  while(1) { 
    if (pulseIn(IR_PIN, HIGH) >= BIT_START) break; // スタートビットを検知したら解析プロセスへ
    if (state == PTT_ON) timer_job();              // PTTがオン状態ならタイマー処理
    }

  ir_code = get_ir_key();                          // 赤外線リモコンコード読み込み

  if (ir_code != -1) do_response(ir_code);         // コードが正常ならキーに応じた処理を行う
  else tone(BUZZ_PIN,200,100);                     // コードが異常ならブザーを鳴動する
  
  delay(300);                                      // リモコンの多重押しを防止するためのウェイト

}

// ------- スイッチ押下割り込み

void button_in() {

    if (state == PTT_ON) {
      tone(BUZZ_PIN,400,1000);                    // ブザーを1秒鳴動
      state = PTT_OFF;                            // PTTがオンならオフに切り替え  
      ptt_control();
      }
      
}

// ------ タイマー処理

void timer_job()
{

  if (timer1.check() == 1) {

    timer1.reset();                             // タイマーをリセット

    ptt_timer = ptt_timer + 1;                  // タイマーカウンターに1加算（経過秒を示す）

    if (ptt_timer == ALERM_TIME) {              // 5分経過したか？
      tone(BUZZ_PIN,400,100);                   // 5分経過を示すためにブザーを鳴動
      delay(200);
      tone(BUZZ_PIN,400,100);
      delay(200);
      tone(BUZZ_PIN,400,100);
      delay(200);
      tone(BUZZ_PIN,400,100);
      delay(200);
      tone(BUZZ_PIN,400,100);
      }
    else if (ptt_timer > FORCE_OFF_TIME) {        // 10分経過したか？
      ptt_timer = 0;                              // タイマーカウンターをクリア
      tone(BUZZ_PIN,400,1000);                    // ブザーを1秒鳴動
      state = PTT_OFF;                            // PTTをオフに切り替え  
      ptt_control();                              // PTT制御
      }

    }      

}

// ----- PTT制御

void ptt_control()
{

  if (state == PTT_ON) {
    digitalWrite(PTT_PIN, HIGH);         // PTTをオン
    digitalWrite(LED_PIN, HIGH);         // LEDを点灯
  }
  else {
    digitalWrite(PTT_PIN, LOW);          // PTTをオフ
    digitalWrite(LED_PIN, LOW);          // LEDを消灯
    }    
  
}

// ----- 赤外線リモコンコード読み込み

int get_ir_key() 
{

  int pulse[IR_BIT_LENGTH];                        // 赤外線パルス格納配列
  int bits[IR_BIT_LENGTH];                         // 赤外線パルスを2進数に変換して格納する配列
 
  read_pulse(pulse);                               // 赤外線パルス読み込み

  if (pulse_to_bits(pulse, bits) != 0) return(-1); // 赤外線パルスを2進数に変換
  if (RemoteVerify(bits) != 0) return(-1);         // カスタムコードをチェック

  return(bits_to_int(bits));                       // コードをIntegerに変換して関数の戻り値とする

}

 
// ----- 赤外線パルス読み込み

void read_pulse(int pulse[])
{

  int i;
  
  for (i = 0; i < IR_BIT_LENGTH; i = i + 1) {
    pulse[i] = pulseIn(IR_PIN, HIGH);              // 各ビットごとのパルス長を読み込む
    }

}
 
// ------- 赤外線パルスを2進数に変換

int pulse_to_bits(int pulse[], int bits[])
{
  
  int i;
  
  for (i = 0; i < IR_BIT_LENGTH; i = i + 1)  {
    
    if (pulse[i] > BIT_1) bits[i] = 1;             // パルス長が1の長さ
    else if (pulse[i] > BIT_0) bits[i] = 0;        // パルス長が0の長さ
    else return(-1);                               // パルス長が不正

  }
  
  return(0);

}
 
// ------- カスタムコードチェック 

int RemoteVerify(int bits[])
{
  
  int i;
  int result = 0;
  int seed = 1;
   
  for (i = 0 ; i < 15 ; i = i + 1) {      // 2進数の配列をIntegerに変換       
    if(bits[i] == 1) result = result + seed;
    seed = seed * 2;
    }

  if (result != VERIFY_CODE) return(-1);          // メーカーが違う場合はエラー

  return(0);

}
 
// ------- 2進数の配列を整数値に変換
 
int bits_to_int(int bits[])
{

  int i;
  int result = 0;
  int seed = 1;
  
//  for (i = (IR_BIT_LENGTH - FIRST_LAST_BIT); i < IR_BIT_LENGTH ; i = i + 1) {
  for (i = 16 ; i < IR_BIT_LENGTH ; i = i + 1) {
    if (bits[i] == 1) result = result + seed;
    seed = seed * 2;
    }

  return(result);

}
 
// ------- 赤外線リモコンのキーに応じた処理を行う

void do_response(unsigned int key)
{  

  switch (key) {
    
    case KEY0_CODE:                         // リモコンの0キーが押されたか？

      if (state == PTT_ON) {                // PTTがオンか？
        state = PTT_OFF;                    // PTTがオンならオフに切り替え  
        }
      else {
        state = PTT_ON;                     // PTTがオフならオンに切り替え
        ptt_timer = 0;
        timer1.reset();                     // タイマーをリセット
        }

      tone(BUZZ_PIN,400,100);               // ブザーを正常音で鳴動する
      
      ptt_control();
    
      break;
      
    default:                                // 0以外のキーが押された

      tone(BUZZ_PIN,200,100);               // ブザーをエラー音で鳴動する

      break;

  }

}
